/*
 *
 *   OpenChanfix v2.0 
 *
 *   Channel Re-op Service Module for Hybrid 7.1 or later.
 *   Copyright (C) 2003 Thomas Carlsson and Joost Vunderink.
 *   See http://www.garion.org/ocf/ for more information.
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 1, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */





/*
Handling of incoming messages to the chanfix client.

All incoming messages to chanfix are converted from the single-line string
to the message_data struct. The members of this struct are:

type     - the type of message. One of the MSG_* defines.
numeric  - if the type == MSG_NUMERIC, this int tells us what the numeric is.
nick     - the nick of the sender, if applicable
user     - the username of the sender, if applicable
host     - the host of the sender, if applicable
server   - the servername, if applicable
data     - the data sent

Possible messages:
PING :chanfix.carnique.nl
:chanfix.carnique.nl 401 C Beige :No such nick/channel
:cfgaar!garion@chan.fix PRIVMSG C :chanfix #test
:chanfix.carnique.nl 001 C :Welcome to the CNQNet Internet Relay Chat Network C
:chanfix.carnique.nl 254 C 2 :channels formed
:chanfix.carnique.nl NOTICE C :*** Spoofing your IP. congrats.

*/

#define MSG_UNKNOWN			0
#define MSG_NUMERIC			1
#define MSG_PING			2
#define MSG_PRIVMSG			3
#define MSG_CHANMSG			4
#define MSG_CTCP			5
#define MSG_KILLED			6
#define MSG_RECON_TOO_FAST	7
#define MSG_NUM_001			8




/* This struct holds data about a message sent by the ircd
 * to the chanfix client. */
struct message_data {
	int type;
	int numeric;
	char nick[NICKLEN + 1];
	char user[USERLEN + 1];
	char host[HOSTLEN + 1];
	char server[HOSTLEN + 1];
	char channel[CHANNELLEN + 1];
	char data[IRCLINELEN + 1];
};

/* Separates readBuf into 1 message per line and processes
 * each message separately */		
int process_received_messages(const char* readBuf, int length);

/* Converts a single message into a struct message_data */
void get_message_data(char *msg, struct message_data *msgdata);

/* Converts a single message, which is a PRIVMSG to me,
 * into a struct message_data */
void get_privmsg_data(char *msg, struct message_data *msgdata);

/* Converts a single message, which is a PRIVMSG to a channel I'm on,
 * into a struct message_data */
void get_chanmsg_data(char *msg, struct message_data *msgdata);

/* Takes action on this message */
void process_single_message(struct message_data *md);

/* Takes action on this PRIVMSG */
void process_privmsg(struct message_data *md);

/* Takes action on this CHANMSG */
void process_chanmsg(struct message_data *md);

/* Takes action when getting killed */
void process_killed(struct message_data *md);

void send_command_list(char* nick);


typedef void (*process_cmd_t)(struct message_data *, struct command_data *);
extern process_cmd_t process_cmd_functions[];

/* Sends a notice to all logged in users with notice flas
 * flag_required. */
void send_notice_to_loggedin(int flag_required, char *pattern, ...);

/* Sends a notice to all logged in users with notice flas
 * flag_required, except to user_p (who is probably the origin. */
void send_notice_to_loggedin_except(struct ocf_loggedin_user *user_p, 
									int flag_required, char *pattern, ...);

/* Checks all logged in users, and sends the message to each logged
 * in user with username <username>. This is usually just 1 nick, but
 * it can happen that a user has logged in with two clients. */
void send_notice_to_loggedin_username(const char* username, char *pattern, ...);

/* And below here, the process functions which do all the work. */

void process_cmd_unknown(struct message_data *md, struct command_data *cd);
void process_cmd_help(struct message_data *md, struct command_data *cd);
void process_cmd_score(struct message_data *md, struct command_data *cd);
void process_cmd_chanfix(struct message_data *md, struct command_data *cd);
void process_cmd_login(struct message_data *md, struct command_data *cd);
void process_cmd_logout(struct message_data *md, struct command_data *cd);
void process_cmd_passwd(struct message_data *md, struct command_data *cd);
void process_cmd_history(struct message_data *md, struct command_data *cd);
void process_cmd_info(struct message_data *md, struct command_data *cd);
void process_cmd_oplist(struct message_data *md, struct command_data *cd);
void process_cmd_addnote(struct message_data *md, struct command_data *cd);
void process_cmd_delnote(struct message_data *md, struct command_data *cd);
void process_cmd_whois(struct message_data *md, struct command_data *cd);
void process_cmd_addflag(struct message_data *md, struct command_data *cd);
void process_cmd_delflag(struct message_data *md, struct command_data *cd);
void process_cmd_addhost(struct message_data *md, struct command_data *cd);
void process_cmd_delhost(struct message_data *md, struct command_data *cd);
void process_cmd_adduser(struct message_data *md, struct command_data *cd);
void process_cmd_deluser(struct message_data *md, struct command_data *cd);
void process_cmd_chpass(struct message_data *md, struct command_data *cd);
void process_cmd_who(struct message_data *md, struct command_data *cd);
void process_cmd_umode(struct message_data *md, struct command_data *cd);
void process_cmd_status(struct message_data *md, struct command_data *cd);
void process_cmd_set(struct message_data *md, struct command_data *cd);
void process_cmd_block(struct message_data *md, struct command_data *cd);
void process_cmd_unblock(struct message_data *md, struct command_data *cd);
void process_cmd_rehash(struct message_data *md, struct command_data *cd);
void process_cmd_check(struct message_data *md, struct command_data *cd);
void process_cmd_opnicks(struct message_data *md, struct command_data *cd);
void process_cmd_cscore(struct message_data *md, struct command_data *cd);



void process_cmd_wrong_syntax(struct message_data *md, struct command_data *cd);

void process_cmd_score_existing_channel(struct message_data *md, struct command_data *cd, struct Channel* chptr, int compact);
void process_cmd_score_nonexisting_channel(struct message_data *md, struct command_data *cd, int compact);
void process_cmd_score_nick(struct message_data *md, struct command_data *cd, int compact);
void process_cmd_score_hostmask(struct message_data *md, struct command_data *cd, int compact);
